@@ -6,14 +6,11 @@ This should be pretty lightweight and not depend on other mypy code (other than
 import json
 
 from typing import Any
+from typing_extensions import Final
 
 from mypy.ipc import IPCBase
 
-MYPY = False
-if MYPY:
-    from typing_extensions import Final
-
-DEFAULT_STATUS_FILE = '.dmypy.json'  # type: Final
+DEFAULT_STATUS_FILE: Final = ".dmypy.json"
 
 
 def receive(connection: IPCBase) -> Any:
@@ -27,8 +24,8 @@ def receive(connection: IPCBase) -> Any:
         raise OSError("No data received")
     try:
         data = json.loads(bdata.decode('utf8'))
-    except Exception:
-        raise OSError("Data received is not valid JSON")
+    except Exception as e:
+        raise OSError("Data received is not valid JSON") from e
     if not isinstance(data, dict):
-        raise OSError("Data received is not a dict (%s)" % str(type(data)))
+        raise OSError(f"Data received is not a dict ({type(data)})")
     return data
