@@ -22,17 +22,17 @@ def solve_constraints(vars: List[TypeVarId], constraints: List[Constraint],
     pick AnyType.
     """
     # Collect a list of constraints for each type variable.
-    cmap = defaultdict(list)  # type: Dict[TypeVarId, List[Constraint]]
+    cmap: Dict[TypeVarId, List[Constraint]] = defaultdict(list)
     for con in constraints:
         cmap[con.type_var].append(con)
 
-    res = []  # type: List[Optional[Type]]
+    res: List[Optional[Type]] = []
 
     # Solve each type variable separately.
     for tvar in vars:
-        bottom = None  # type: Optional[Type]
-        top = None  # type: Optional[Type]
-        candidate = None  # type: Optional[Type]
+        bottom: Optional[Type] = None
+        top: Optional[Type] = None
+        candidate: Optional[Type] = None
 
         # Process each constraint separately, and calculate the lower and upper
         # bounds based on constraints. Note that we assume that the constraint
