@@ -5,7 +5,7 @@ NOTE: These must not be accessed from mypy.nodes or mypy.types to avoid import
       since these may assume that MROs are ready.
 """
 
-from typing import cast, Optional, List, Sequence, Set, Iterable, TypeVar, Dict, Tuple, Any
+from typing import cast, Optional, List, Sequence, Set, Iterable, TypeVar, Dict, Tuple, Any, Union
 from typing_extensions import Type as TypingType
 import itertools
 import sys
@@ -14,8 +14,8 @@ from mypy.types import (
     TupleType, Instance, FunctionLike, Type, CallableType, TypeVarLikeType, Overloaded,
     TypeVarType, UninhabitedType, FormalArgument, UnionType, NoneType,
     AnyType, TypeOfAny, TypeType, ProperType, LiteralType, get_proper_type, get_proper_types,
-    copy_type, TypeAliasType, TypeQuery, ParamSpecType,
-    ENUM_REMOVED_PROPS
+    TypeAliasType, TypeQuery, ParamSpecType, Parameters, UnpackType, TypeVarTupleType,
+    ENUM_REMOVED_PROPS,
 )
 from mypy.nodes import (
     FuncBase, FuncItem, FuncDef, OverloadedFuncDef, TypeInfo, ARG_STAR, ARG_STAR2, ARG_POS,
@@ -23,10 +23,11 @@ from mypy.nodes import (
 )
 from mypy.maptype import map_instance_to_supertype
 from mypy.expandtype import expand_type_by_instance, expand_type
+from mypy.copytype import copy_type
 
 from mypy.typevars import fill_typevars
 
-from mypy import state
+from mypy.state import state
 
 
 def is_recursive_pair(s: Type, t: Type) -> bool:
@@ -42,7 +43,22 @@ def tuple_fallback(typ: TupleType) -> Instance:
     info = typ.partial_fallback.type
     if info.fullname != 'builtins.tuple':
         return typ.partial_fallback
-    return Instance(info, [join_type_list(typ.items)])
+    items = []
+    for item in typ.items:
+        proper_type = get_proper_type(item)
+        if isinstance(proper_type, UnpackType):
+            unpacked_type = get_proper_type(proper_type.type)
+            if isinstance(unpacked_type, TypeVarTupleType):
+                items.append(unpacked_type.upper_bound)
+            elif isinstance(unpacked_type, TupleType):
+                # TODO: might make sense to do recursion here to support nested unpacks
+                # of tuple constants
+                items.extend(unpacked_type.items)
+            else:
+                raise NotImplementedError
+        else:
+            items.append(item)
+    return Instance(info, [join_type_list(items)])
 
 
 def type_object_type_from_function(signature: FunctionLike,
@@ -107,6 +123,10 @@ def class_callable(init_type: CallableType, info: TypeInfo, type_type: Instance,
     explicit_type = init_ret_type if is_new else orig_self_type
     if (
         isinstance(explicit_type, (Instance, TupleType))
+        # We have to skip protocols, because it can be a subtype of a return type
+        # by accident. Like `Hashable` is a subtype of `object`. See #11799
+        and isinstance(default_ret_type, Instance)
+        and not default_ret_type.type.is_protocol
         # Only use the declared return type from __new__ or declared self in __init__
         # if it is actually returning a subtype of what we would return otherwise.
         and is_subtype(explicit_type, default_ret_type, ignore_type_params=True)
@@ -268,7 +288,7 @@ def erase_to_bound(t: Type) -> Type:
     return t
 
 
-def callable_corresponding_argument(typ: CallableType,
+def callable_corresponding_argument(typ: Union[CallableType, Parameters],
                                     model: FormalArgument) -> Optional[FormalArgument]:
     """Return the argument a function that corresponds to `model`"""
 
@@ -295,15 +315,41 @@ def callable_corresponding_argument(typ: CallableType,
     return by_name if by_name is not None else by_pos
 
 
-def is_simple_literal(t: ProperType) -> bool:
-    """
-    Whether a type is a simple enough literal to allow for fast Union simplification
+def simple_literal_value_key(t: ProperType) -> Optional[Tuple[str, ...]]:
+    """Return a hashable description of simple literal type.
 
-    For now this means enum or string
+    Return None if not a simple literal type.
+
+    The return value can be used to simplify away duplicate types in
+    unions by comparing keys for equality. For now enum, string or
+    Instance with string last_known_value are supported.
     """
-    return isinstance(t, LiteralType) and (
-            t.fallback.type.is_enum or t.fallback.type.fullname == 'builtins.str'
-    )
+    if isinstance(t, LiteralType):
+        if t.fallback.type.is_enum or t.fallback.type.fullname == 'builtins.str':
+            assert isinstance(t.value, str)
+            return 'literal', t.value, t.fallback.type.fullname
+    if isinstance(t, Instance):
+        if t.last_known_value is not None and isinstance(t.last_known_value.value, str):
+            return 'instance', t.last_known_value.value, t.type.fullname
+    return None
+
+
+def simple_literal_type(t: Optional[ProperType]) -> Optional[Instance]:
+    """Extract the underlying fallback Instance type for a simple Literal"""
+    if isinstance(t, Instance) and t.last_known_value is not None:
+        t = t.last_known_value
+    if isinstance(t, LiteralType):
+        return t.fallback
+    return None
+
+
+def is_simple_literal(t: ProperType) -> bool:
+    """Fast way to check if simple_literal_value_key() would return a non-None value."""
+    if isinstance(t, LiteralType):
+        return t.fallback.type.is_enum or t.fallback.type.fullname == 'builtins.str'
+    if isinstance(t, Instance):
+        return t.last_known_value is not None and isinstance(t.last_known_value.value, str)
+    return False
 
 
 def make_simplified_union(items: Sequence[Type],
@@ -324,10 +370,17 @@ def make_simplified_union(items: Sequence[Type],
 
     Note: This must NOT be used during semantic analysis, since TypeInfos may not
           be fully initialized.
+
     The keep_erased flag is used for type inference against union types
     containing type variables. If set to True, keep all ErasedType items.
+
+    The contract_literals flag indicates whether we need to contract literal types
+    back into a sum type. Set it to False when called by try_expanding_sum_type_
+    to_union().
     """
     items = get_proper_types(items)
+
+    # Step 1: expand all nested unions
     while any(isinstance(typ, UnionType) for typ in items):
         all_items: List[ProperType] = []
         for typ in items:
@@ -337,23 +390,32 @@ def make_simplified_union(items: Sequence[Type],
                 all_items.append(typ)
         items = all_items
 
+    # Step 2: remove redundant unions
+    simplified_set = _remove_redundant_union_items(items, keep_erased)
+
+    # Step 3: If more than one literal exists in the union, try to simplify
+    if contract_literals and sum(isinstance(item, LiteralType) for item in simplified_set) > 1:
+        simplified_set = try_contracting_literals_in_union(simplified_set)
+
+    return UnionType.make_union(simplified_set, line, column)
+
+
+def _remove_redundant_union_items(items: List[ProperType], keep_erased: bool) -> List[ProperType]:
     from mypy.subtypes import is_proper_subtype
 
     removed: Set[int] = set()
-    seen: Set[Tuple[str, str]] = set()
+    seen: Set[Tuple[str, ...]] = set()
 
     # NB: having a separate fast path for Union of Literal and slow path for other things
     # would arguably be cleaner, however it breaks down when simplifying the Union of two
-    # different enum types as try_expanding_enum_to_union works recursively and will
+    # different enum types as try_expanding_sum_type_to_union works recursively and will
     # trigger intermediate simplifications that would render the fast path useless
     for i, item in enumerate(items):
         if i in removed:
             continue
         # Avoid slow nested for loop for Union of Literal of strings/enums (issue #9169)
-        if is_simple_literal(item):
-            assert isinstance(item, LiteralType)
-            assert isinstance(item.value, str)
-            k = (item.value, item.fallback.type.fullname)
+        k = simple_literal_value_key(item)
+        if k is not None:
             if k in seen:
                 removed.add(i)
                 continue
@@ -369,15 +431,27 @@ def make_simplified_union(items: Sequence[Type],
             seen.add(k)
             if safe_skip:
                 continue
-        # Keep track of the truishness info for deleted subtypes which can be relevant
+
+        # Keep track of the truthiness info for deleted subtypes which can be relevant
         cbt = cbf = False
         for j, tj in enumerate(items):
-            # NB: we don't need to check literals as the fast path above takes care of that
             if (
-                    i != j
-                    and not is_simple_literal(tj)
-                    and is_proper_subtype(tj, item, keep_erased_types=keep_erased)
-                    and is_redundant_literal_instance(item, tj)  # XXX?
+                i == j
+                # avoid further checks if this item was already marked redundant.
+                or j in removed
+                # if the current item is a simple literal then this simplification loop can
+                # safely skip all other simple literals as two literals will only ever be
+                # subtypes of each other if they are equal, which is already handled above.
+                # However, if the current item is not a literal, it might plausibly be a
+                # supertype of other literals in the union, so we must check them again.
+                # This is an important optimization as is_proper_subtype is pretty expensive.
+                or (k is not None and is_simple_literal(tj))
+            ):
+                continue
+            # actual redundancy checks
+            if (
+                is_redundant_literal_instance(item, tj)  # XXX?
+                and is_proper_subtype(tj, item, keep_erased_types=keep_erased)
             ):
                 # We found a redundant item in the union.
                 removed.add(j)
@@ -389,13 +463,7 @@ def make_simplified_union(items: Sequence[Type],
         elif not item.can_be_false and cbf:
             items[i] = true_or_false(item)
 
-    simplified_set = [items[i] for i in range(len(items)) if i not in removed]
-
-    # If more than one literal exists in the union, try to simplify
-    if (contract_literals and sum(isinstance(item, LiteralType) for item in simplified_set) > 1):
-        simplified_set = try_contracting_literals_in_union(simplified_set)
-
-    return UnionType.make_union(simplified_set, line, column)
+    return [items[i] for i in range(len(items)) if i not in removed]
 
 
 def _get_type_special_method_bool_ret_type(t: Type) -> Optional[Type]:
@@ -565,7 +633,7 @@ def try_getting_str_literals(expr: Expression, typ: Type) -> Optional[List[str]]
     Otherwise, returns None.
 
     Specifically, this function is guaranteed to return a list with
-    one or more strings if one one the following is true:
+    one or more strings if one of the following is true:
 
     1. 'expr' is a StrExpr
     2. 'typ' is a LiteralType containing a string
@@ -607,7 +675,7 @@ def try_getting_literals_from_type(typ: Type,
                                    target_literal_type: TypingType[T],
                                    target_fullname: str) -> Optional[List[T]]:
     """If the given expression or type corresponds to a Literal or
-    union of Literals where the underlying values corresponds to the given
+    union of Literals where the underlying values correspond to the given
     target type, returns a list of those underlying values. Otherwise,
     returns None.
     """
@@ -759,7 +827,7 @@ def try_contracting_literals_in_union(types: Sequence[Type]) -> List[ProperType]
                 if fullname not in sum_types:
                     sum_types[fullname] = (set(get_enum_values(typ.fallback))
                                            if typ.fallback.type.is_enum
-                                           else set((True, False)),
+                                           else {True, False},
                                            [])
                 literals, indexes = sum_types[fullname]
                 literals.discard(typ.value)
