@@ -1,7 +1,7 @@
 """Transform class definitions from the mypy AST form to IR."""
 
 from abc import abstractmethod
-from typing import Callable, List, Optional, Tuple
+from typing import Callable, List, Optional, Set, Tuple
 from typing_extensions import Final
 
 from mypy.nodes import (
@@ -214,7 +214,10 @@ class ExtClassBuilder(ClassBuilder):
             self.builder.init_final_static(lvalue, value, self.cdef.name)
 
     def finalize(self, ir: ClassIR) -> None:
-        generate_attr_defaults(self.builder, self.cdef, self.skip_attr_default)
+        attrs_with_defaults, default_assignments = find_attr_initializers(
+            self.builder, self.cdef, self.skip_attr_default)
+        ir.attrs_with_defaults.update(attrs_with_defaults)
+        generate_attr_defaults_init(self.builder, self.cdef, default_assignments)
         create_ne_from_eq(self.builder, self.cdef)
 
 
@@ -524,9 +527,11 @@ def add_non_ext_class_attr(builder: IRBuilder,
             attr_to_cache.append((lvalue, object_rprimitive))
 
 
-def generate_attr_defaults(builder: IRBuilder, cdef: ClassDef,
-                           skip: Optional[Callable[[str, AssignmentStmt], bool]] = None) -> None:
-    """Generate an initialization method for default attr values (from class vars).
+def find_attr_initializers(builder: IRBuilder,
+                           cdef: ClassDef,
+                           skip: Optional[Callable[[str, AssignmentStmt], bool]] = None,
+                           ) -> Tuple[Set[str], List[AssignmentStmt]]:
+    """Find initializers of attributes in a class body.
 
     If provided, the skip arg should be a callable which will return whether
     to skip generating a default for an attribute.  It will be passed the name of
@@ -534,7 +539,9 @@ def generate_attr_defaults(builder: IRBuilder, cdef: ClassDef,
     """
     cls = builder.mapper.type_to_ir[cdef.info]
     if cls.builtin_base:
-        return
+        return set(), []
+
+    attrs_with_defaults = set()
 
     # Pull out all assignments in classes in the mro so we can initialize them
     # TODO: Support nested statements
@@ -558,10 +565,30 @@ def generate_attr_defaults(builder: IRBuilder, cdef: ClassDef,
                 if skip is not None and skip(name, stmt):
                     continue
 
+                attr_type = cls.attr_type(name)
+
+                # If the attribute is initialized to None and type isn't optional,
+                # doesn't initialize it to anything (special case for "# type:" comments).
+                if isinstance(stmt.rvalue, RefExpr) and stmt.rvalue.fullname == 'builtins.None':
+                    if (not is_optional_type(attr_type) and not is_object_rprimitive(attr_type)
+                            and not is_none_rprimitive(attr_type)):
+                        continue
+
+                attrs_with_defaults.add(name)
                 default_assignments.append(stmt)
 
+    return attrs_with_defaults, default_assignments
+
+
+def generate_attr_defaults_init(builder: IRBuilder,
+                                cdef: ClassDef,
+                                default_assignments: List[AssignmentStmt]) -> None:
+    """Generate an initialization method for default attr values (from class vars)."""
     if not default_assignments:
         return
+    cls = builder.mapper.type_to_ir[cdef.info]
+    if cls.builtin_base:
+        return
 
     with builder.enter_method(cls, '__mypyc_defaults_setup', bool_rprimitive):
         self_var = builder.self()
@@ -571,15 +598,11 @@ def generate_attr_defaults(builder: IRBuilder, cdef: ClassDef,
             if not stmt.is_final_def and not is_constant(stmt.rvalue):
                 builder.warning('Unsupported default attribute value', stmt.rvalue.line)
 
-            # If the attribute is initialized to None and type isn't optional,
-            # don't initialize it to anything.
             attr_type = cls.attr_type(lvalue.name)
-            if isinstance(stmt.rvalue, RefExpr) and stmt.rvalue.fullname == 'builtins.None':
-                if (not is_optional_type(attr_type) and not is_object_rprimitive(attr_type)
-                        and not is_none_rprimitive(attr_type)):
-                    continue
             val = builder.coerce(builder.accept(stmt.rvalue), attr_type, stmt.line)
-            builder.add(SetAttr(self_var, lvalue.name, val, -1))
+            init = SetAttr(self_var, lvalue.name, val, -1)
+            init.mark_as_initializer()
+            builder.add(init)
 
         builder.add(Return(builder.true()))
 
@@ -588,11 +611,11 @@ def check_deletable_declaration(builder: IRBuilder, cl: ClassIR, line: int) -> N
     for attr in cl.deletable:
         if attr not in cl.attributes:
             if not cl.has_attr(attr):
-                builder.error('Attribute "{}" not defined'.format(attr), line)
+                builder.error(f'Attribute "{attr}" not defined', line)
                 continue
             for base in cl.mro:
                 if attr in base.property_types:
-                    builder.error('Cannot make property "{}" deletable'.format(attr), line)
+                    builder.error(f'Cannot make property "{attr}" deletable', line)
                     break
             else:
                 _, base = cl.attr_details(attr)
